﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Windows.Forms;
using ff14bot.Enums;
using ff14bot.Managers;

namespace DestinyPlugin
{
    public partial class Gui : Form
    {
        private Dictionary<int, string> _locations;

        public Gui()
        {
            InitializeComponent();
        }

        private Dictionary<int, string> Locations
        {
            get { return _locations ?? (_locations = BuildKnownLocations()); }
        }

        private void Gui_Load(object sender, EventArgs e)
        {
            Text = string.Format("Destiny V{0}", Destiny.InternalVersion);
            SetComboBoxes();
            SetNumericUpDowns();
            SetCheckBoxes();
            BossProgressTrackBar.Value = (int)Settings.Instance.BossPercentRequired;
            BossFatePercentageLabel.Text = string.Format("Engage Boss Fates when progress is above {0}", BossProgressTrackBar.Value);
        }

        private void SetComboBoxes()
        {
            var combatRoutineList = BuildCombatRoutineList().ToList();
            PreferedRoutine_ComboBox.DataSource = combatRoutineList;
            if (!string.IsNullOrEmpty(Settings.Instance.PreferedRoutine) &&
                combatRoutineList.Contains(Settings.Instance.PreferedRoutine))
            {
                PreferedRoutine_ComboBox.SelectedItem = Settings.Instance.PreferedRoutine;
            }

            Levels01_10ComboBox.ValueMember = "Key";
            Levels01_10ComboBox.DisplayMember = "Value";
            Levels01_10ComboBox.DataSource = new BindingSource(Locations, null);
            Levels01_10ComboBox.SelectedValue = Settings.Instance.Levels01_10Location;

            Levels10_15ComboBox.ValueMember = "Key";
            Levels10_15ComboBox.DisplayMember = "Value";
            Levels10_15ComboBox.DataSource = new BindingSource(Locations, null);
            Levels10_15ComboBox.SelectedValue = Settings.Instance.Levels10_15Location;
			
			

            Levels15_20ComboBox.ValueMember = "Key";
            Levels15_20ComboBox.DisplayMember = "Value";
            Levels15_20ComboBox.DataSource = new BindingSource(Locations, null);
            Levels15_20ComboBox.SelectedValue = Settings.Instance.Levels15_20Location;

            Levels20_25ComboBox.ValueMember = "Key";
            Levels20_25ComboBox.DisplayMember = "Value";
            Levels20_25ComboBox.DataSource = new BindingSource(Locations, null);
            Levels20_25ComboBox.SelectedValue = Settings.Instance.Levels20_25Location;

            Levels25_30ComboBox.ValueMember = "Key";
            Levels25_30ComboBox.DisplayMember = "Value";
            Levels25_30ComboBox.DataSource = new BindingSource(Locations, null);
            Levels25_30ComboBox.SelectedValue = Settings.Instance.Levels25_30Location;

            Levels30_35ComboBox.ValueMember = "Key";
            Levels30_35ComboBox.DisplayMember = "Value";
            Levels30_35ComboBox.DataSource = new BindingSource(Locations, null);
            Levels30_35ComboBox.SelectedValue = Settings.Instance.Levels30_35Location;

            Levels35_40ComboBox.ValueMember = "Key";
            Levels35_40ComboBox.DisplayMember = "Value";
            Levels35_40ComboBox.DataSource = new BindingSource(Locations, null);
            Levels35_40ComboBox.SelectedValue = Settings.Instance.Levels35_40Location;

            Levels40_45ComboBox.ValueMember = "Key";
            Levels40_45ComboBox.DisplayMember = "Value";
            Levels40_45ComboBox.DataSource = new BindingSource(Locations, null);
            Levels40_45ComboBox.SelectedValue = Settings.Instance.Levels40_45Location;

            Levels45_50ComboBox.ValueMember = "Key";
            Levels45_50ComboBox.DisplayMember = "Value";
            Levels45_50ComboBox.DataSource = new BindingSource(Locations, null);
            Levels45_50ComboBox.SelectedValue = Settings.Instance.Levels45_50Location;
        }

        private void SetNumericUpDowns()
        {
            Archer_BardUpDown.Value = Settings.Instance.Archer_BardGearSet;
            Gladiator_PaladinUpDown.Value = Settings.Instance.Gladiator_PaladinGearSet;
            Lancer_DragoonUpDown.Value = Settings.Instance.Lancer_DragoonGearSet;
            Marauder_WarriorUpDown.Value = Settings.Instance.Marauder_WarriorGearSet;
            Pugilist_MonkUpDown.Value = Settings.Instance.Pugilist_MonkGearSet;
            Rogue_NinjaUpDown.Value = Settings.Instance.Rogue_NinjaGearSet;
            Conjurer_WhiteMageUpDown.Value = Settings.Instance.Conjurer_WhiteMageGearSet;
            Thaumaturge_BlackMageUpDown.Value = Settings.Instance.Thaumaturge_BlackMageGearSet;
            ArcanistUpDown.Value = Settings.Instance.ArcanistGearSet;
            SummonerUpDown.Value = Settings.Instance.SummonerGearSet;
            ScholarUpDown.Value = Settings.Instance.ScholarGearSet;
            AstrologianUpDown.Value = Settings.Instance.AstrologianGearSet;
            DarkKnightUpDown.Value = Settings.Instance.DarkKnightGearSet;
            MachinistUpDown.Value = Settings.Instance.MachinistGearSet;
            MaxLevelUpDown.Value = Settings.Instance.MaxLevel;
            MinLevelUpDown.Value = Settings.Instance.MinLevel;
        }

        private void SetCheckBoxes()
        {
            MonsterSlayingCheckBox.Checked = Settings.Instance.MonsterSlayingEnabled;
            BossCheckBox.Checked = Settings.Instance.BossEnabled;
            EscortCheckBox.Checked = Settings.Instance.EscortEnabled;
            LevelRequirementsCheckBox.Checked = Settings.Instance.LevelCheck;
            SetHomePointcheckBox.Checked = Settings.Instance.SetAetheryteAsHome;
        }

        private static Dictionary<int, string> BuildKnownLocations()
        {
            var textInfo = new CultureInfo("en-US", false).TextInfo;
            return WorldManager.AvailableLocations.OrderBy(x => x.Name)
                .ToDictionary(x => (int) x.AetheryteId, x => textInfo.ToTitleCase(x.Name));
        }

        private static IEnumerable<string> BuildCombatRoutineList()
        {
            var retval = new HashSet<string>();
            foreach (
                var routine in RoutineManager.AllRoutines.Select(x => x.Name).Where(x => x != "InvalidRoutineWrapper"))
            {
                var index = routine.IndexOf('[');
                retval.Add(index > 0 ? routine.Substring(0, index) : routine);
            }

            retval.Add(string.Empty);

            return retval;
        }

        #region ComboBox Selection Change Commited

        private void PreferedRoutine_ComboBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            Settings.Instance.PreferedRoutine = PreferedRoutine_ComboBox.SelectedItem.ToString();
            Settings.Instance.Save();
        }

        private void Levels01_10ComboBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            Destiny.ZoneLevels[1] = (int) Levels01_10ComboBox.SelectedValue;
            Destiny.ZoneLevels[2] = (int) Levels01_10ComboBox.SelectedValue;
            Destiny.ZoneLevels[3] = (int) Levels01_10ComboBox.SelectedValue;
            Destiny.ZoneLevels[4] = (int) Levels01_10ComboBox.SelectedValue;
            Destiny.ZoneLevels[5] = (int) Levels01_10ComboBox.SelectedValue;
            Destiny.ZoneLevels[6] = (int) Levels01_10ComboBox.SelectedValue;
            Destiny.ZoneLevels[7] = (int) Levels01_10ComboBox.SelectedValue;
            Destiny.ZoneLevels[8] = (int) Levels01_10ComboBox.SelectedValue;
            Destiny.ZoneLevels[9] = (int) Levels01_10ComboBox.SelectedValue;
            Settings.Instance.Levels01_10Location = (int) Levels01_10ComboBox.SelectedValue;
            Settings.Instance.Save();
        }

        private void Levels10_15ComboBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            Destiny.ZoneLevels[10] = (int) Levels10_15ComboBox.SelectedValue;
            Destiny.ZoneLevels[11] = (int) Levels10_15ComboBox.SelectedValue;
            Destiny.ZoneLevels[12] = (int) Levels10_15ComboBox.SelectedValue;
            Destiny.ZoneLevels[13] = (int) Levels10_15ComboBox.SelectedValue;
            Destiny.ZoneLevels[14] = (int) Levels10_15ComboBox.SelectedValue;
            Settings.Instance.Levels10_15Location = (int) Levels10_15ComboBox.SelectedValue;
            Settings.Instance.Save();
        }

        private void Levels15_20ComboBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            Destiny.ZoneLevels[15] = (int) Levels15_20ComboBox.SelectedValue;
            Destiny.ZoneLevels[16] = (int) Levels15_20ComboBox.SelectedValue;
            Destiny.ZoneLevels[17] = (int) Levels15_20ComboBox.SelectedValue;
            Destiny.ZoneLevels[18] = (int) Levels15_20ComboBox.SelectedValue;
            Destiny.ZoneLevels[19] = (int) Levels15_20ComboBox.SelectedValue;
            Settings.Instance.Levels15_20Location = (int) Levels15_20ComboBox.SelectedValue;
            Settings.Instance.Save();
        }

        private void Levels20_25ComboBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            Destiny.ZoneLevels[20] = (int) Levels20_25ComboBox.SelectedValue;
            Destiny.ZoneLevels[21] = (int) Levels20_25ComboBox.SelectedValue;
            Destiny.ZoneLevels[22] = (int) Levels20_25ComboBox.SelectedValue;
            Destiny.ZoneLevels[23] = (int) Levels20_25ComboBox.SelectedValue;
            Destiny.ZoneLevels[24] = (int) Levels20_25ComboBox.SelectedValue;
            Settings.Instance.Levels20_25Location = (int) Levels20_25ComboBox.SelectedValue;
            Settings.Instance.Save();
        }

        private void Levels25_30ComboBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            Destiny.ZoneLevels[25] = (int) Levels25_30ComboBox.SelectedValue;
            Destiny.ZoneLevels[26] = (int) Levels25_30ComboBox.SelectedValue;
            Destiny.ZoneLevels[27] = (int) Levels25_30ComboBox.SelectedValue;
            Destiny.ZoneLevels[28] = (int) Levels25_30ComboBox.SelectedValue;
            Destiny.ZoneLevels[29] = (int) Levels25_30ComboBox.SelectedValue;
            Settings.Instance.Levels25_30Location = (int) Levels25_30ComboBox.SelectedValue;
            Settings.Instance.Save();
        }

        private void Levels30_35ComboBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            Destiny.ZoneLevels[30] = (int) Levels30_35ComboBox.SelectedValue;
            Destiny.ZoneLevels[31] = (int) Levels30_35ComboBox.SelectedValue;
            Destiny.ZoneLevels[32] = (int) Levels30_35ComboBox.SelectedValue;
            Destiny.ZoneLevels[33] = (int) Levels30_35ComboBox.SelectedValue;
            Destiny.ZoneLevels[34] = (int) Levels30_35ComboBox.SelectedValue;
            Settings.Instance.Levels30_35Location = (int) Levels30_35ComboBox.SelectedValue;
            Settings.Instance.Save();
        }

        private void Levels35_40ComboBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            Destiny.ZoneLevels[35] = (int) Levels35_40ComboBox.SelectedValue;
            Destiny.ZoneLevels[36] = (int) Levels35_40ComboBox.SelectedValue;
            Destiny.ZoneLevels[37] = (int) Levels35_40ComboBox.SelectedValue;
            Destiny.ZoneLevels[38] = (int) Levels35_40ComboBox.SelectedValue;
            Destiny.ZoneLevels[39] = (int) Levels35_40ComboBox.SelectedValue;
            Settings.Instance.Levels35_40Location = (int) Levels35_40ComboBox.SelectedValue;
            Settings.Instance.Save();
        }

        private void Levels40_45ComboBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            Destiny.ZoneLevels[40] = (int) Levels40_45ComboBox.SelectedValue;
            Destiny.ZoneLevels[41] = (int) Levels40_45ComboBox.SelectedValue;
            Destiny.ZoneLevels[42] = (int) Levels40_45ComboBox.SelectedValue;
            Destiny.ZoneLevels[43] = (int) Levels40_45ComboBox.SelectedValue;
            Destiny.ZoneLevels[44] = (int) Levels40_45ComboBox.SelectedValue;
            Settings.Instance.Levels40_45Location = (int) Levels40_45ComboBox.SelectedValue;
            Settings.Instance.Save();
        }

        private void Levels45_50ComboBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            Destiny.ZoneLevels[45] = (int) Levels45_50ComboBox.SelectedValue;
            Destiny.ZoneLevels[46] = (int) Levels45_50ComboBox.SelectedValue;
            Destiny.ZoneLevels[47] = (int) Levels45_50ComboBox.SelectedValue;
            Destiny.ZoneLevels[48] = (int) Levels45_50ComboBox.SelectedValue;
            Destiny.ZoneLevels[49] = (int) Levels45_50ComboBox.SelectedValue;
            Destiny.ZoneLevels[50] = (int) Levels45_50ComboBox.SelectedValue;
            Settings.Instance.Levels45_50Location = (int) Levels45_50ComboBox.SelectedValue;
            Settings.Instance.Save();
        }

        #endregion

        #region UpDown Value Changed

        private void Archer_BardUpDown_ValueChanged(object sender, EventArgs e)
        {
            Destiny.GearSets[ClassJobType.Archer] = (int) Archer_BardUpDown.Value;
            Destiny.GearSets[ClassJobType.Bard] = (int) Archer_BardUpDown.Value;
            Settings.Instance.Archer_BardGearSet = (int) Archer_BardUpDown.Value;
            Settings.Instance.Save();
        }

        private void Gladiator_PaladinUpDown_ValueChanged(object sender, EventArgs e)
        {
            Destiny.GearSets[ClassJobType.Gladiator] = (int) Gladiator_PaladinUpDown.Value;
            Destiny.GearSets[ClassJobType.Paladin] = (int) Gladiator_PaladinUpDown.Value;
            Settings.Instance.Gladiator_PaladinGearSet = (int) Gladiator_PaladinUpDown.Value;
            Settings.Instance.Save();
        }

        private void Lancer_DragoonUpDown_ValueChanged(object sender, EventArgs e)
        {
            Destiny.GearSets[ClassJobType.Lancer] = (int) Lancer_DragoonUpDown.Value;
            Destiny.GearSets[ClassJobType.Dragoon] = (int) Lancer_DragoonUpDown.Value;
            Settings.Instance.Lancer_DragoonGearSet = (int) Lancer_DragoonUpDown.Value;
            Settings.Instance.Save();
        }

        private void Marauder_WarriorUpDown_ValueChanged(object sender, EventArgs e)
        {
            Destiny.GearSets[ClassJobType.Marauder] = (int) Marauder_WarriorUpDown.Value;
            Destiny.GearSets[ClassJobType.Warrior] = (int) Marauder_WarriorUpDown.Value;
            Settings.Instance.Marauder_WarriorGearSet = (int) Marauder_WarriorUpDown.Value;
            Settings.Instance.Save();
        }

        private void Pugilist_MonkUpDown_ValueChanged(object sender, EventArgs e)
        {
            Destiny.GearSets[ClassJobType.Pugilist] = (int) Pugilist_MonkUpDown.Value;
            Destiny.GearSets[ClassJobType.Monk] = (int) Pugilist_MonkUpDown.Value;
            Settings.Instance.Pugilist_MonkGearSet = (int) Pugilist_MonkUpDown.Value;
            Settings.Instance.Save();
        }

        private void Rogue_NinjaUpDown_ValueChanged(object sender, EventArgs e)
        {
            Destiny.GearSets[ClassJobType.Rogue] = (int) Rogue_NinjaUpDown.Value;
            Destiny.GearSets[ClassJobType.Ninja] = (int) Rogue_NinjaUpDown.Value;
            Settings.Instance.Rogue_NinjaGearSet = (int) Rogue_NinjaUpDown.Value;
            Settings.Instance.Save();
        }

        private void Conjurer_WhiteMageUpDown_ValueChanged(object sender, EventArgs e)
        {
            Destiny.GearSets[ClassJobType.Conjurer] = (int) Conjurer_WhiteMageUpDown.Value;
            Destiny.GearSets[ClassJobType.WhiteMage] = (int) Conjurer_WhiteMageUpDown.Value;
            Settings.Instance.Conjurer_WhiteMageGearSet = (int) Conjurer_WhiteMageUpDown.Value;
            Settings.Instance.Save();
        }

        private void Thaumaturge_BlackMageUpDown_ValueChanged(object sender, EventArgs e)
        {
            Destiny.GearSets[ClassJobType.Thaumaturge] = (int) Thaumaturge_BlackMageUpDown.Value;
            Destiny.GearSets[ClassJobType.BlackMage] = (int) Thaumaturge_BlackMageUpDown.Value;
            Settings.Instance.Thaumaturge_BlackMageGearSet = (int) Thaumaturge_BlackMageUpDown.Value;
            Settings.Instance.Save();
        }

        private void ArcanistUpDown_ValueChanged(object sender, EventArgs e)
        {
            Destiny.GearSets[ClassJobType.Arcanist] = (int) ArcanistUpDown.Value;
            Settings.Instance.ArcanistGearSet = (int) ArcanistUpDown.Value;
            Settings.Instance.Save();
        }

        private void SummonerUpDown_ValueChanged(object sender, EventArgs e)
        {
            Destiny.GearSets[ClassJobType.Summoner] = (int) SummonerUpDown.Value;
            Settings.Instance.SummonerGearSet = (int) SummonerUpDown.Value;
            Settings.Instance.Save();
        }

        private void ScholarUpDown_ValueChanged(object sender, EventArgs e)
        {
            Destiny.GearSets[ClassJobType.Scholar] = (int) ScholarUpDown.Value;
            Settings.Instance.ScholarGearSet = (int) ScholarUpDown.Value;
            Settings.Instance.Save();
        }

        private void AstrologianUpDown_ValueChanged(object sender, EventArgs e)
        {
            //Destiny.GearSets[ClassJobType.Astrologian] = (int)AstrologianUpDown.Value;
            Settings.Instance.AstrologianGearSet = (int) AstrologianUpDown.Value;
            Settings.Instance.Save();
        }

        private void DarkKnightUpDown_ValueChanged(object sender, EventArgs e)
        {
            //Destiny.GearSets[ClassJobType.DarkKnight] = (int)DarkKnightUpDown.Value;
            Settings.Instance.DarkKnightGearSet = (int) DarkKnightUpDown.Value;
            Settings.Instance.Save();
        }

        private void MachinistUpDown_ValueChanged(object sender, EventArgs e)
        {
            //Destiny.GearSets[ClassJobType.Machinist] = (int)MachinistUpDown.Value;
            Settings.Instance.MachinistGearSet = (int) MachinistUpDown.Value;
            Settings.Instance.Save();
        }

        private void MaxLevelUpDown_ValueChanged(object sender, EventArgs e)
        {
            Settings.Instance.MaxLevel = (int) MaxLevelUpDown.Value;
            Settings.Instance.Save();
        }

        private void MinLevelUpDown_ValueChanged(object sender, EventArgs e)
        {
            Settings.Instance.MinLevel = (int) MinLevelUpDown.Value;
            Settings.Instance.Save();
        }

        #endregion

        #region CheckBoxes Checked

        private void MonsterSlayingCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            Settings.Instance.MonsterSlayingEnabled = MonsterSlayingCheckBox.Checked;
            Settings.Instance.Save();
        }

        private void BossCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            Settings.Instance.BossEnabled = BossCheckBox.Checked;
            Settings.Instance.Save();
        }

        private void EscortCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            Settings.Instance.EscortEnabled = EscortCheckBox.Checked;
            Settings.Instance.Save();
        }

        private void LevelRequirementsCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            Settings.Instance.LevelCheck = LevelRequirementsCheckBox.Checked;
            Settings.Instance.Save();
        }
        private void SetHomePointcheckBox_CheckedChanged(object sender, EventArgs e)
        {
            Settings.Instance.SetAetheryteAsHome = SetHomePointcheckBox.Checked;
            Settings.Instance.Save();
        }
        #endregion

        private void BossProgressTrackBar_Scroll(object sender, EventArgs e)
        {
            Settings.Instance.BossPercentRequired = BossProgressTrackBar.Value;
            Settings.Instance.Save();
            BossFatePercentageLabel.Text = string.Format("Engage Boss Fates when progress is above {0}", BossProgressTrackBar.Value);
        }

        
    }
}