﻿using System.ComponentModel;
using System.Configuration;
using System.IO;
using ff14bot.Helpers;
using Newtonsoft.Json;

// ReSharper disable InconsistentNaming

namespace DestinyPlugin
{
    public class Settings : JsonSettings
    {
        [JsonIgnore]
        private static Settings _instance;
        // ReSharper disable once UnusedParameter.Local
        // ReSharper disable once MemberCanBePrivate.Global
        public Settings(string filename)
            : base(Path.Combine(CharacterSettingsDirectory, string.Format("DestinyV{0}.json", Destiny.InternalVersion)))
        {
        }

        public static Settings Instance
        {
            get { return _instance ?? (_instance = new Settings("Settings")); }
        }

        #region FateBot Settings

        [Setting]
        public string PreferedRoutine { get; set; }
        [Setting]
        [DefaultValue(true)]
        public bool BossEnabled { get; set; }
        [Setting]
        [DefaultValue(true)]
        public bool EscortEnabled { get; set; }
        [Setting]
        [DefaultValue(true)]
        public bool MonsterSlayingEnabled { get; set; }
        [Setting]
        [DefaultValue(true)]
        public bool LevelCheck { get; set; }
        [Setting]
        [DefaultValue(3)]
        public int MaxLevel { get; set; }
        [Setting]
        [DefaultValue(5)]
        public int MinLevel { get; set; }
        [Setting]
        [DefaultValue(10f)]
        public float BossPercentRequired { get; set; }



        #endregion

        #region FateBot Farming Locations
        [Setting]
        [DefaultValue(true)]
        public bool SetAetheryteAsHome { get; set; }

        [Setting]
        [DefaultValue(52)]
        public int Levels01_10Location { get; set; }

        [Setting]
        [DefaultValue(52)]
        public int Levels10_15Location { get; set; }

        [Setting]
        [DefaultValue(14)]
        public int Levels15_20Location { get; set; }

        [Setting]
        [DefaultValue(4)]
        public int Levels20_25Location { get; set; }

        [Setting]
        [DefaultValue(19)]
        public int Levels25_30Location { get; set; }

        [Setting]
        [DefaultValue(20)]
        public int Levels30_35Location { get; set; }

        [Setting]
        [DefaultValue(23)]
        public int Levels35_40Location { get; set; }

        [Setting]
        [DefaultValue(23)]
        public int Levels40_45Location { get; set; }

        [Setting]
        [DefaultValue(22)]
        public int Levels45_50Location { get; set; }

        #endregion

        #region GearSet Values

        [Setting]
        [DefaultValue(0)]
        public int Archer_BardGearSet { get; set; }

        [Setting]
        [DefaultValue(0)]
        public int Gladiator_PaladinGearSet { get; set; }

        [Setting]
        [DefaultValue(0)]
        public int Lancer_DragoonGearSet { get; set; }

        [Setting]
        [DefaultValue(0)]
        public int Marauder_WarriorGearSet { get; set; }

        [Setting]
        [DefaultValue(0)]
        public int Pugilist_MonkGearSet { get; set; }

        [Setting]
        [DefaultValue(0)]
        public int Rogue_NinjaGearSet { get; set; }

        [Setting]
        [DefaultValue(0)]
        public int Conjurer_WhiteMageGearSet { get; set; }

        [Setting]
        [DefaultValue(0)]
        public int Thaumaturge_BlackMageGearSet { get; set; }

        [Setting]
        [DefaultValue(0)]
        public int ArcanistGearSet { get; set; }

        [Setting]
        [DefaultValue(0)]
        public int SummonerGearSet { get; set; }

        [Setting]
        [DefaultValue(0)]
        public int ScholarGearSet { get; set; }

        [Setting]
        [DefaultValue(0)]
        public int AstrologianGearSet { get; set; }

        [Setting]
        [DefaultValue(0)]
        public int DarkKnightGearSet { get; set; }

        [Setting]
        [DefaultValue(0)]
        public int MachinistGearSet { get; set; }

        #endregion
    }
}